@@ -1,6 +1,6 @@
 /*
  *-----------------------------------------------------------------------------
- * $Id: reftools.c 2195 2012-12-19 11:39:32Z schindlerf $
+ * $Id$
  *
  * Project: EOxServer <http://eoxserver.org>
  * Authors: Stephan Krause <stephan.krause@eox.at>
@@ -31,20 +31,36 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <gdal/gdal.h>
-#include <gdal/gdal_alg.h>
-#include <gdal/gdalwarper.h>
-#include <gdal/ogr_srs_api.h>
-#include <gdal/cpl_string.h>
+#include <gdal.h>
+#include <gdal_alg.h>
+#include <gdalwarper.h>
+#include <ogr_srs_api.h>
+#include <cpl_string.h>
+
 
 /******************************************************************************/
+/* NOTE: define -DUSE_GDAL_EOX_EXTENSIONS to compile the EOX extended version */
+/******************************************************************************/
+
+/* GDAL Transformer methods */
+#define METHOD_GCP 1  
+#define METHOD_TPS 2  
+#define METHOD_TPS_LSQ 3  
+
+/* check that the required extensions are available */
+#ifdef USE_GDAL_EOX_EXTENSIONS
+#ifndef GDAL_USE_TPS2 
+#error "The GDAL library does not provide required extensions!"
+#endif 
+#endif
+
 /******************************************************************************/
 
 typedef struct {
     size_t n_points;
     double *x;
     double *y;
-} eoxs_footprint;
+} EOXS_FOOTPRINT;
 
 typedef struct {
     int srid;
@@ -52,25 +68,77 @@ typedef struct {
     double miny;
     double maxx;
     double maxy;
-} eoxs_subset;
+} EOXS_SUBSET;
 
 typedef struct {
     int x_off;
     int y_off;
     int x_size;
     int y_size;
-} eoxs_rect;
+} EOXS_RECT;
 
 typedef struct {
-    size_t x_size;
-    size_t y_size;
+    int x_size;
+    int y_size;
     double geotransform[6];
-} eoxs_image_info;
+} EOXS_IMAGE_INFO;
 
 /******************************************************************************/
 /******************************************************************************/
 
-void eoxs_destroy_footprint(eoxs_footprint *fp) {
+/* check method and order */
+CPLErr check_method_and_order(int method, int order)
+{ 
+    if (METHOD_GCP==method) 
+    { 
+        if (( order < 0 )||( order > 3 ))
+        { 
+            CPLError(CE_Failure,CPLE_IllegalArg,"Invalid polynomial order! ORDER=%d",order); 
+            return CE_Failure;
+        }
+    } 
+#ifdef USE_GDAL_EOX_EXTENSIONS
+    else if ((METHOD_TPS==method)||(METHOD_TPS_LSQ==method ))
+    { 
+        if (( order < -1 )||( order > 3 ))
+        { 
+            CPLError(CE_Failure,CPLE_IllegalArg,"Invalid TPS augmenting polynomial order! ORDER=%d",order); 
+            return CE_Failure;
+        }
+    }
+#else 
+    else if ((METHOD_TPS==method)||(1!=order))
+    { 
+        CPLError(CE_Failure,CPLE_IllegalArg,"Invalid TPS augmenting polynomial order! ORDER=%d",order) ; 
+        return CE_Failure;
+    }
+    else if (METHOD_TPS_LSQ==method)
+    { 
+        CPLError(CE_Failure,CPLE_IllegalArg,"TPS_LSQ method not supported!") ;
+        return CE_Failure;
+    } 
+#endif 
+    else 
+    { 
+        CPLError(CE_Failure,CPLE_IllegalArg,"Invalid method! METHOD=%d",method) ;
+        return CE_Failure;
+    } 
+
+    return CE_None ; 
+} 
+
+
+/* return 1 for extended or 0 for baseline version of GDAL */ 
+int eoxs_is_extended( void ) 
+{ 
+#ifdef USE_GDAL_EOX_EXTENSIONS
+    return 1 ; 
+#else 
+    return 0 ; 
+#endif 
+} 
+
+void eoxs_destroy_footprint(EOXS_FOOTPRINT *fp) {
     free(fp->x);
     free(fp->y);
     free(fp);
@@ -81,24 +149,129 @@ void eoxs_free_string(char* str)
     free(str);
 }
 
-void *eoxs_create_referenceable_grid_transformer(GDALDatasetH ds, int order) {
+/******************************************************************************/
+
+void *eoxs_create_referenceable_grid_transformer(GDALDatasetH ds, int method, int order)
+{
     int gcp_count;
     const GDAL_GCP *gcps;
 
+    if ( check_method_and_order(method,order) != CE_None ) return NULL ; 
+
     if (!ds) return NULL;
     
     gcp_count = GDALGetGCPCount(ds);
     gcps = GDALGetGCPs(ds);
 
-    if (order == -1) {
-        return GDALCreateTPSTransformer(gcp_count, gcps, FALSE);
-    }
-    else {
+#ifdef DEBUG 
+    printf("GDAL Referenceable Transformer:\n") ; 
+    printf("method = %d\n",method) ; 
+    printf("order = %d\n",order) ; 
+#endif 
+
+    if ( METHOD_GCP == method ) 
+    { 
         return GDALCreateGCPTransformer(gcp_count, gcps, order, FALSE);
+    } 
+    else if ( METHOD_TPS == method ) 
+    { 
+#ifdef USE_GDAL_EOX_EXTENSIONS
+        return GDALCreateTPS2TransformerExt(gcp_count, gcps, FALSE, order);
+#else 
+        return GDALCreateTPSTransformer(gcp_count, gcps, FALSE);
+#endif 
+    } 
+#ifdef USE_GDAL_EOX_EXTENSIONS
+    else if ( METHOD_TPS_LSQ == method ) 
+    { 
+        return GDALCreateTPS2TransformerLSQGrid(gcp_count, gcps, FALSE, order, 0, 0);
+    } 
+#endif 
+    else 
+    { 
+        return NULL ; 
     }
 }
 
-CPLErr eoxs_calculate_footprint(GDALDatasetH ds, int order, eoxs_footprint **out_footprint) {
+/******************************************************************************/
+
+void *eoxs_create_gen_img_proj_transformer(
+    GDALDatasetH ds_src, // mandatory 
+    const char * srs_wkt_src, // can be NULL 
+    GDALDatasetH ds_dst,     // can be NULL 
+    const char * srs_wkt_dst, // can be NULL (if ds_dst provided)
+    int method, 
+    int order)
+{
+    void *transformer = NULL ;
+    char ** topt = NULL ; 
+
+#ifdef DEBUG 
+    printf("GDAL Gen.Img. Transformer:\n") ; 
+    printf("method = %d\n",method) ; 
+    printf("order = %d\n",order) ; 
+#endif 
+
+    if ( check_method_and_order(method,order) != CE_None ) return NULL ; 
+
+    // SET TRANSFOMER OPTIONS
+    
+    // destination projection WKT
+    if ( srs_wkt_src ) 
+        topt = CSLSetNameValue( topt, "SRC_SRS", srs_wkt_src );
+    
+    // source projection WKT
+    if ( srs_wkt_dst ) 
+        topt = CSLSetNameValue( topt, "DST_SRS", srs_wkt_dst ); 
+
+    // use GCPs from the source dataset (TRUE set by default)
+    //topt = CSLSetNameValue( topt, "GCPS_OK", "TRUE" ); 
+
+    // method specific options 
+    if (METHOD_GCP==method) 
+    { 
+        topt = CSLSetNameValue( topt, "METHOD", "GCP_POLYNOMIAL" ); // set method to GCP polynomial interpolation
+
+        if ( order > 0 ) 
+            topt = CSLSetNameValue( topt, "MAX_GCP_ORDER", CPLSPrintf("%d",order) ); // set max. polymomial order
+    } 
+#ifdef USE_GDAL_EOX_EXTENSIONS
+    else if ((METHOD_TPS==method)||(METHOD_TPS_LSQ==method))
+    { 
+        topt = CSLSetNameValue( topt, "METHOD", "GCP_TPS2" ); // set method to GCP Thin Plate Spline v2 interpolation 
+        topt = CSLSetNameValue( topt, "TPS2_AP_ORDER", CPLSPrintf("%d",order) ); // augmenting polynomial order 
+    
+        if (METHOD_TPS_LSQ==method)
+        { 
+            topt = CSLSetNameValue( topt, "TPS2_LSQ_GRID", "1" ); // use alternative rectangular grid 
+            topt = CSLSetNameValue( topt, "TPS2_LSQ_GRID_NX", "0" ); // x grid size (0 means default)
+            topt = CSLSetNameValue( topt, "TPS2_LSQ_GRID_NY", "0" ); // y grid size (0 means default)
+        } 
+    } 
+#else
+    else if (METHOD_TPS==method)
+    { 
+        topt = CSLSetNameValue( topt, "METHOD", "GCP_TPS" ); // set method to GCP Thin Plate Spline interpolation 
+    } 
+#endif 
+    else return NULL ; 
+
+    // CREATE TRANSFOMER 
+    transformer = GDALCreateGenImgProjTransformer2(
+        ds_src,  // source dataset 
+        ds_dst,  // destination dataset
+        topt     // trasformer options
+    ); 
+
+    // DISCARD TRANSFOMER OPTIONS 
+    CSLDestroy( topt );
+
+    return transformer ; 
+} 
+
+/******************************************************************************/
+
+CPLErr eoxs_calculate_footprint(GDALDatasetH ds, int method, int order, EOXS_FOOTPRINT **out_footprint) {
     void *transformer;
     int x_size, y_size;
     double *x, *y, *z;
@@ -149,7 +322,7 @@ CPLErr eoxs_calculate_footprint(GDALDatasetH ds, int order, eoxs_footprint **out
     x_size = GDALGetRasterXSize(ds);
     y_size = GDALGetRasterYSize(ds);
 
-    transformer = eoxs_create_referenceable_grid_transformer(ds, order);
+    transformer = eoxs_create_referenceable_grid_transformer(ds, method, order);
 
     if (!transformer) {
         if (CPLGetLastErrorMsg() == NULL) {
@@ -200,7 +373,7 @@ CPLErr eoxs_calculate_footprint(GDALDatasetH ds, int order, eoxs_footprint **out
     free(success);
     GDALDestroyTransformer(transformer);
 
-    *out_footprint = malloc(sizeof(eoxs_footprint));
+    *out_footprint = malloc(sizeof(EOXS_FOOTPRINT));
     (*out_footprint)->n_points = n_points;
     (*out_footprint)->x = x;
     (*out_footprint)->y = y;
@@ -222,14 +395,14 @@ CPLErr eoxs_calculate_footprint(GDALDatasetH ds, int order, eoxs_footprint **out
     return CE_None;
 }
 
-CPLErr eoxs_get_footprint_wkt(GDALDatasetH ds, int order, char **out_wkt) {
-    eoxs_footprint *fp;
+CPLErr eoxs_get_footprint_wkt(GDALDatasetH ds, int method, int order, char **out_wkt) {
+    EOXS_FOOTPRINT *fp;
     char buffer[512];
     int i, maxlen;
     CPLErr ret;
     *out_wkt = NULL;
 
-    if ((ret = eoxs_calculate_footprint(ds, order, &fp)) != CE_None) {
+    if ((ret = eoxs_calculate_footprint(ds, method, order, &fp)) != CE_None) {
         return ret;
     }
 
@@ -293,7 +466,7 @@ void eoxs_get_intermediate_point_count(
     int *n_y,
     int ds_x_size,
     int ds_y_size,
-    eoxs_subset *subset,
+    EOXS_SUBSET *subset,
     void *transformer,
     OGRCoordinateTransformationH ct
 ) {
@@ -325,7 +498,7 @@ void eoxs_get_intermediate_point_count(
     *n_y = (int) ceil((eoxs_array_max(4, y) - eoxs_array_min(4, y)) / dist);
 }
 
-CPLErr eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, int order, eoxs_rect *out_rect) {
+CPLErr eoxs_rect_from_subset(GDALDatasetH ds, EOXS_SUBSET *subset, int method, int order, EOXS_RECT *out_rect) {
     void *transformer;
     
     OGRSpatialReferenceH gcp_srs, subset_srs;
@@ -382,7 +555,7 @@ CPLErr eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, int order, eo
     ds_x_size = GDALGetRasterXSize(ds);
     ds_y_size = GDALGetRasterYSize(ds);
     
-    transformer = eoxs_create_referenceable_grid_transformer(ds, order);
+    transformer = eoxs_create_referenceable_grid_transformer(ds, method, order);
     
     if (!transformer) {
         if (CPLGetLastErrorMsg() == NULL) {
@@ -482,7 +655,13 @@ CPLErr eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, int order, eo
     return CE_None;
 }
 
-CPLErr eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int srid) {
+CPLErr eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename,
+                            int srid, // TODO: make it work with srs_wkt 
+                            GDALResampleAlg eResampleAlg, 
+                            double dfWarpMemoryLimit, 
+                            double dfMaxError,
+                            int method, int order)
+{
     GDALDatasetH vrt_ds;
     OGRSpatialReferenceH dst_srs;
     char *dst_srs_wkt;
@@ -490,6 +669,12 @@ CPLErr eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int
     
     void *transformer;
     GDALWarpOptions *warp_options;
+
+    CPLErr ret ; 
+    int x_size,y_size;
+    double geotransform[6];
+
+    if ( check_method_and_order(method,order) != CE_None ) return CE_Failure; 
     
     if (!ds) {
         CPLError(CE_Failure, CPLE_ObjectNull, "No dataset passed.");
@@ -507,14 +692,12 @@ CPLErr eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int
         free_dst_srs_wkt = FALSE;
     }
     
-    transformer = GDALCreateGenImgProjTransformer(
+    transformer = eoxs_create_gen_img_proj_transformer( 
         ds,                       // source dataset
         NULL,                     // source projection WKT -> read from dataset 
         NULL,                     // destination dataset
         dst_srs_wkt,              // destination projection WKT
-        TRUE,                     // use GCPs
-        0.0,                      // ignored
-        1                         // use interpolation order 1
+        method, order 
     );
     
     if (!transformer) {
@@ -524,59 +707,139 @@ CPLErr eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int
         if (free_dst_srs_wkt) free(dst_srs_wkt);
         return CE_Failure;
     }
+
+    // guess the size of the created image
+
+    ret = GDALSuggestedWarpOutput(ds, GDALGenImgProjTransform, 
+                transformer, geotransform, &x_size, &y_size );
+
+    if ( ret != CE_None ) 
+    {
+        GDALDestroyGenImgProjTransformer(transformer);
+        return ret ;  
+    }
+
+    // update the transformer to include the output geo-transform 
+    
+    GDALSetGenImgProjTransformerDstGeoTransform( transformer, geotransform ); 
+
+    // set warp options 
     
     warp_options = GDALCreateWarpOptions();
-    //warp_options->eResampleAlg = GRA_NearestNeighbour;
-    warp_options->pTransformerArg = transformer;
-    
-    vrt_ds = GDALAutoCreateWarpedVRT(
-        ds,                       // source dataset,
-        NULL,                     // source projection WKT
-        dst_srs_wkt,              // destination projection WKT,
-        GRA_NearestNeighbour,     // resample algorithm
-        0.0,                      // choose exact calculation
-        warp_options              // warp options
-    );
-    
-    if (!vrt_ds) {
-        if (CPLGetLastErrorMsg() == NULL) {
-            CPLError(CE_Failure, CPLE_FileIO, "Failed to created warped VRT.");
-        }
-        GDALDestroyGenImgProjTransformer(transformer);
-        GDALDestroyWarpOptions(warp_options);
-        if (free_dst_srs_wkt) free(dst_srs_wkt);
-        return CE_Failure;
-    }
-    
-    GDALSetDescription(vrt_ds, vrt_filename);
-    
-    // clean up
-    GDALClose(vrt_ds);
-    GDALDestroyGenImgProjTransformer(transformer);
-    GDALDestroyWarpOptions(warp_options);
-    if (free_dst_srs_wkt) free(dst_srs_wkt);
-    
-    return CE_None;
-}
 
+    warp_options->dfWarpMemoryLimit = dfWarpMemoryLimit ;   // warp memory limit 
+    warp_options->eResampleAlg = eResampleAlg ;             // resampling method
+    warp_options->pfnTransformer = GDALGenImgProjTransform ;// specific transform function
+    warp_options->pTransformerArg = transformer;            // pointer to the transfomer 
+    warp_options->hSrcDS = ds ;                             // source dataset 
+
+    { 
+        int i , j , nb ; 
+
+        // bands setup
+        warp_options->nBandCount = nb = GDALGetRasterCount( ds );
+        warp_options->panSrcBands = (int*)CPLMalloc(sizeof(int) * nb );
+        warp_options->panDstBands = (int*)CPLMalloc(sizeof(int) * nb );
+
+        for( i = 0; i < nb ; i++ )
+        {
+            warp_options->panSrcBands[i] = i+1;
+            warp_options->panDstBands[i] = i+1;
+        }
+
+        // handle no-data values alpha bands
+        for( i = 0; i < nb ; i++ )
+        { 
+            GDALRasterBandH band = GDALGetRasterBand( ds, i+1 );
+            int has_noda_value = FALSE ; 
+            double v_nodata = GDALGetRasterNoDataValue( band, &has_noda_value ) ; 
+
+            if ( has_noda_value ) 
+            { 
+                // if not yet allocated allocate the no-data vectors
+            
+                if( NULL == warp_options->padfSrcNoDataReal )
+                { 
+                    warp_options->padfSrcNoDataImag = (double*)CPLCalloc(nb,sizeof(double)) ; 
+                    warp_options->padfSrcNoDataReal = (double*)CPLMalloc(sizeof(double)*nb) ; 
+
+                    //NOTE: the 'magic constant' taken from the GDAL sources 
+                    for ( j = 0 ; j < nb ; ++j ) 
+                        warp_options->padfSrcNoDataReal[j] = -1.1e20;
+                } 
+
+                warp_options->padfSrcNoDataReal[i] = v_nodata ;
+            } 
+
+            if ( GDALGetRasterColorInterpretation(band) == GCI_AlphaBand )
+            { 
+                warp_options->nSrcAlphaBand = i+1 ; 
+                warp_options->nDstAlphaBand = i+1 ; 
+            } 
+        } 
+    } 
+
+    // approximating transformation (desired as it makes warping really faster)
+    if ( 0 < dfMaxError ) 
+    { 
+        warp_options->pTransformerArg =         // pointer to the transfomer
+                GDALCreateApproxTransformer( 
+                    warp_options->pfnTransformer,
+                    warp_options->pTransformerArg, 
+                    dfMaxError ) ; 
+                    
+        warp_options->pfnTransformer = GDALApproxTransform; // specific transform function
+
+        // make sure the original transformer gets destroyed
+        GDALApproxTransformerOwnsSubtransformer(
+                warp_options->pTransformerArg, TRUE );
+    }    
+
+    // create the rectified (warped) VRT
+    // NOTE: VRT dataset steels ownership of the warp_options->pfnTransformer !!!
+    vrt_ds = GDALCreateWarpedVRT( ds, // source dataset
+            x_size, y_size,     // image size 
+            geotransform,       // geotransformation matrix
+            warp_options ) ;    // warp options 
+
+    if ( vrt_ds ) // SUCCESS 
+    {
+        // set some additional metadata
+        GDALSetProjection( vrt_ds, dst_srs_wkt );
+        GDALSetDescription( vrt_ds, vrt_filename );
+
+        // close the dataset 
+        GDALClose(vrt_ds); // NOTE: GDALClose() destroys the transformers!!!
+
+        // NOTE: GDALClose() commits the actual file write. Check for any possible I/O errors!
+        ret = CPLGetLastErrorType() ; 
+    } 
+    else    // FAILURE 
+    {
+        if (CPLGetLastErrorMsg() == NULL)
+            CPLError(CE_Failure, CPLE_FileIO, "Failed to created warped VRT.");
+
+        ret = CE_Failure;
+    }
+   
+    // clean-up the mess 
+    
+    GDALDestroyWarpOptions( warp_options );
+    if (free_dst_srs_wkt) free(dst_srs_wkt);
+
+    return ret ;
+}
 
 /* Thin wrapper for GDALSuggestedWarpOutput which allows the setting of the order. */
 CPLErr eoxs_suggested_warp_output(GDALDatasetH ds, 
                                   const char *src_wkt, /* can be NULL */
                                   const char *dst_wkt,
-                                  int order,
-                                  eoxs_image_info* out) {
+                                  int method, int order,
+                                  EOXS_IMAGE_INFO* out) {
 
     CPLErr ret;
-    void *transformer = GDALCreateGenImgProjTransformer(
-        ds,
-        src_wkt,
-        NULL,
-        dst_wkt,
-        TRUE,
-        0.0,
-        order
-    );
+    void *transformer = eoxs_create_gen_img_proj_transformer( 
+        ds, src_wkt, NULL, dst_wkt, method, order ) ; 
 
     if (!transformer) {
         return CE_Failure;
@@ -585,7 +848,7 @@ CPLErr eoxs_suggested_warp_output(GDALDatasetH ds,
     ret = GDALSuggestedWarpOutput(ds, GDALGenImgProjTransform, 
                                   transformer, 
                                   out->geotransform,
-                                  (int*) &out->x_size, (int*) &out->y_size);
+                                  &out->x_size, &out->y_size);
 
     // clean up
     GDALDestroyTransformer(transformer);
@@ -594,7 +857,7 @@ CPLErr eoxs_suggested_warp_output(GDALDatasetH ds,
 }
 
 
-/* copied from gdalwarper.cpp */
+/* original source copied from gdalwarper.cpp - some mods have been made though */
 CPLErr eoxs_reproject_image(GDALDatasetH hSrcDS,
                             const char *pszSrcWKT, 
                             GDALDatasetH hDstDS,
@@ -602,25 +865,16 @@ CPLErr eoxs_reproject_image(GDALDatasetH hSrcDS,
                             GDALResampleAlg eResampleAlg, 
                             double dfWarpMemoryLimit, 
                             double dfMaxError,
-                            int order) {
+                            int method, int order) 
+{
     
     GDALWarpOptions *psWOptions;
 
 /* -------------------------------------------------------------------- */
 /*      Setup a reprojection based transformer.                         */
 /* -------------------------------------------------------------------- */
-    void *hTransformArg;
-
-    hTransformArg = 
-        GDALCreateGenImgProjTransformer(
-            hSrcDS,
-            pszSrcWKT,
-            hDstDS,
-            pszDstWKT,
-            TRUE,
-            0.0,
-            order
-        );
+    void *hTransformArg = eoxs_create_gen_img_proj_transformer( 
+            hSrcDS, pszSrcWKT, hDstDS, pszDstWKT, method, order ) ;
 
     if( hTransformArg == NULL )
         return CE_Failure;
@@ -631,6 +885,7 @@ CPLErr eoxs_reproject_image(GDALDatasetH hSrcDS,
 /* -------------------------------------------------------------------- */
     psWOptions = GDALCreateWarpOptions();
     psWOptions->eResampleAlg = eResampleAlg;
+    psWOptions->dfWarpMemoryLimit = dfWarpMemoryLimit ;   // warp memory limit 
 
 /* -------------------------------------------------------------------- */
 /*      Set transform.                                                  */
