@@ -1,9 +1,10 @@
 #-------------------------------------------------------------------------------
-# $Id: admin.py 1937 2012-08-07 13:16:47Z schindlerf $
+# $Id$
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
 #          Stephan Meissl <stephan.meissl@eox.at>
+#          Martin Paces <martin.paces@eox.at>
 #
 #-------------------------------------------------------------------------------
 # Copyright (C) 2011 EOX IT Services GmbH
@@ -56,6 +57,8 @@ from eoxserver.core.system import System
 from eoxserver.core.admin import ConfirmationAdmin
 from eoxserver.core.util.timetools import UTCOffsetTimeZoneInfo
 
+
+logger = logging.getLogger(__name__)
 System.init()
 
 # NilValue
@@ -138,6 +141,11 @@ class DataPackageInline(admin.StackedInline):
 class LineageInline(admin.StackedInline):
     model = LineageRecord
 
+class LayerMetadata2DatasetInline(admin.StackedInline):
+    model = RectifiedDatasetRecord.layer_metadata.__getattribute__("through")
+    extra = 1
+    can_delete = True
+
 class RectifiedDatasetAdmin(ConfirmationAdmin):
     #list_display = ('coverage_id', 'eo_id', 'data_package', 'range_type', 'extent')
     fields = ('automatic', 'visible', 'coverage_id', 'eo_id', 'range_type', 'extent', 'eo_metadata', 'data_package', 'data_source', 'lineage')
@@ -148,7 +156,7 @@ class RectifiedDatasetAdmin(ConfirmationAdmin):
     
     ordering = ('coverage_id', )
     search_fields = ('coverage_id', )
-    inlines = (StitchedMosaic2DatasetInline, DatasetSeries2DatasetInline, )
+    inlines = (StitchedMosaic2DatasetInline, DatasetSeries2DatasetInline, LayerMetadata2DatasetInline)
     
     # We need to override the bulk delete function of the admin to make
     # sure the overrode delete() method of EODatasetMixIn is
@@ -208,10 +216,10 @@ admin.site.register(RectifiedDatasetRecord, RectifiedDatasetAdmin)
 
 class ReferenceableDatasetAdmin(ConfirmationAdmin):
     #list_display = ('coverage_id', 'eo_id', 'data_package', 'range_type', 'size_x', 'size_y')
-    fields = ('automatic', 'visible', 'coverage_id', 'eo_id', 'range_type', 'size_x', 'size_y', 'eo_metadata', 'data_package', 'lineage')
-    list_display = ('coverage_id', 'eo_id', 'range_type', 'size_x', 'size_y', 'visible')
+    fields = ('automatic', 'visible', 'coverage_id', 'eo_id', 'range_type', 'extent', 'eo_metadata', 'data_package', 'lineage')
+    list_display = ('coverage_id', 'eo_id', 'range_type', 'extent', 'visible')
     #list_editable = ('data_package', 'range_type', 'extent')
-    list_editable = ('range_type', 'size_x', 'size_y', 'visible')
+    list_editable = ('range_type', 'extent', 'visible')
     list_filter = ('range_type', )
     
     ordering = ('coverage_id', )
@@ -257,9 +265,7 @@ class ReferenceableDatasetAdmin(ConfirmationAdmin):
         if obj is not None and obj.automatic:
             return self.readonly_fields + (
                 'coverage_id', 'eo_id', 'eo_metadata',
-                'lineage', 'data_package', 'size_x', 'size_y',
-                'layer_metadata', 
-            )
+                'lineage', 'data_package', 'extent', 'layer_metadata' )
             
         return self.readonly_fields
     
@@ -330,9 +336,9 @@ class AbstractContainerAdmin(admin.ModelAdmin):
         except obj_to_sync.DoesNotExist:
             pass
     
-    def add_view(self, request, form_url="", extra_context=None):
+    def add_view(self, request, *args, **kwargs):
         try:
-            ret = super(AbstractContainerAdmin, self).add_view(request, form_url, extra_context)
+            ret = super(AbstractContainerAdmin, self).add_view(request, *args, **kwargs)
             self.try_synchronize()
             return ret
         except:
@@ -346,20 +352,22 @@ class AbstractContainerAdmin(admin.ModelAdmin):
             return ret
         except:
             messages.error(request, "Could not change %s" % self.model._meta.verbose_name)
+            raise
             return HttpResponseRedirect("..")
     
-    def changelist_view(self, request, extra_context=None):
+    def changelist_view(self, request, *args, **kwargs):
         try:
-            ret = super(AbstractContainerAdmin, self).changelist_view(request, extra_context)
+            ret = super(AbstractContainerAdmin, self).changelist_view(request, *args, **kwargs)
             self.try_synchronize()
             return ret
         except:
             messages.error(request, "Could not change %s" % self.model._meta.verbose_name)
+            raise
             return HttpResponseRedirect("..")
     
-    def delete_view(self, request, object_id, extra_context=None):
+    def delete_view(self, request, *args, **kwargs):
         try:
-            ret = super(AbstractContainerAdmin, self).delete_view(request, object_id, extra_context)
+            ret = super(AbstractContainerAdmin, self).delete_view(request, *args, **kwargs)
             # TODO: need synchronization here?
             return ret
         except:
@@ -400,7 +408,13 @@ admin.site.register(RectifiedStitchedMosaicRecord, RectifiedStitchedMosaicAdmin)
     def save_model(self, request, obj, form, change):
         raise # TODO"""
 
+class LayerMetadata2DatasetSeriesInline(admin.StackedInline):
+    model = DatasetSeriesRecord.layer_metadata.__getattribute__("through")
+    extra = 1
+    can_delete = True
+
 class DatasetSeriesAdmin(AbstractContainerAdmin):
+    inlines = (LayerMetadata2DatasetSeriesInline,)
     list_display = ('eo_id', 'eo_metadata', )
     list_editable = ('eo_metadata', )
     ordering = ('eo_id', )
@@ -481,7 +495,7 @@ class EOMetadataAdmin(admin.GeoModelAdmin):
                 else:
                     synchronizer.create()
             except:
-                logging.error("Error when synchronizing.")
+                logger.error("Error when synchronizing.")
                 #transaction.rollback()
                 messages.error(request, "Error when synchronizing with file system.")
                 #return
