@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: data.py 1520 2012-03-13 16:47:33Z schindlerf $
+# $Id$
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -28,30 +28,30 @@
 #-------------------------------------------------------------------------------
 
 import os.path
-
-from osgeo import gdal
 import logging
 
-from eoxserver.resources.coverages.metadata import MetadataFormat
-gdal.UseExceptions() # advise GDAL to raise Python exceptions
-
+from eoxserver.contrib import gdal
 from eoxserver.core.system import System
 from eoxserver.core.records import (
-    RecordWrapperInterface, RecordWrapper, RecordWrapperFactoryInterface,
-    RecordWrapperFactory
+    RecordWrapper, RecordWrapperFactoryInterface, RecordWrapperFactory
 )
 from eoxserver.core.exceptions import InternalError
 from eoxserver.backends.cache import CacheFileWrapper
-from eoxserver.resources.coverages.exceptions import EngineError
+from eoxserver.resources.coverages.exceptions import (
+    EngineError, MetadataException
+)
 from eoxserver.resources.coverages.models import (
     DataSource, DataPackage, LocalDataPackage, RemoteDataPackage,
     RasdamanDataPackage, TileIndex
 )
 from eoxserver.resources.coverages.interfaces import (
-    DataSourceInterface, CoverageDataInterface, DataPackageInterface,
-    TileIndexInterface
+    DataSourceInterface, DataPackageInterface, TileIndexInterface
 )
 from eoxserver.resources.coverages.geo import GeospatialMetadata
+from eoxserver.resources.coverages.formats import getFormatRegistry
+
+
+logger = logging.getLogger(__name__)
 
 #-------------------------------------------------------------------------------
 # Data source wrappers
@@ -257,6 +257,12 @@ class DataPackageWrapper(RecordWrapper):
                 )
             )
     
+    def getSourceFormat(self): 
+        """
+        Return the source data file format. 
+        """
+        return str( self.record.source_format )
+
     def getLocation(self):
         """
         Return the location of the data, i.e. an object implementing
@@ -370,7 +376,7 @@ class DataPackageWrapper(RecordWrapper):
                 self._post_metadata_access()
             except:
                 self._post_metadata_access()
-                raise
+                raise MetadataException("")
                 
             return eo_metadata
         else:
@@ -479,6 +485,10 @@ class LocalDataPackageWrapper(DataPackageWrapper):
         )
     }
     
+    def __init__(self, **kwargs):
+        super(LocalDataPackageWrapper, self).__init__(**kwargs)
+        self.source_format = None
+    
     def getDataStructureType(self):
         """
         Returns ``"file"``.
@@ -535,6 +545,14 @@ class LocalDataPackageWrapper(DataPackageWrapper):
         else:
             self.record = record.localdatapackage
     
+    def _set_attrs(self, **kwargs):
+        super(LocalDataPackageWrapper, self)._set_attrs(**kwargs)
+        self.source_format = kwargs.get("source_format")
+        
+        if self.source_format is None:
+            driver_name = "GDAL/" + self.open().GetDriver().ShortName
+            frmt = getFormatRegistry().getFormatsByDriver(driver_name)[0]
+            self.source_format = frmt.mimeType
     
     def _get_query_set(self, query):
         return LocalDataPackage.objects.filter(**query)
@@ -551,7 +569,8 @@ class LocalDataPackageWrapper(DataPackageWrapper):
             data_package_type = LocalDataPackage.DATA_PACKAGE_TYPE,
             data_location = self.location.getRecord(),
             metadata_location = metadata_location_record,
-            metadata_format_name = self.metadata_format_name
+            metadata_format_name = self.metadata_format_name,
+            source_format = self.source_format
         )
         
     def _prepare_metadata_access(self):
@@ -692,13 +711,19 @@ class RemoteDataPackageWrapper(DataPackageWrapper):
         super(RemoteDataPackageWrapper, self)._set_attrs(**kwargs)
         
         self.cache_file = kwargs.get("cache_file")
+        self.source_format = kwargs.get("source_format")
+        
+        if self.source_format is None:
+            driver_name = "GDAL/" + self.open().GetDriver().ShortName
+            frmt = getFormatRegistry().getFormatsByDriver(driver_name)[0]
+            self.source_format = frmt.mimeType
 
     def _get_query(self, fields=None):
         query = super(RemoteDataPackageWrapper, self)._get_query(fields)
         
         if fields is None or "cache_file" in fields:
             if self.cache_file:
-                query["cache_file"] = self.cache_file.getRecord()
+                query["cache_file"] = self.cache_file.getModel()
             else:
                 query["cache_file"] = None
         
@@ -712,7 +737,7 @@ class RemoteDataPackageWrapper(DataPackageWrapper):
         
         
         if self.cache_file:
-            cache_file_record = self.cache_file.getRecord()
+            cache_file_record = self.cache_file.getModel()
         else:
             cache_file_record = None
         
@@ -726,7 +751,8 @@ class RemoteDataPackageWrapper(DataPackageWrapper):
             data_location = self.location.getRecord(),
             metadata_location = metadata_location_record,
             metadata_format_name = self.metadata_format_name,
-            cache_file = cache_file_record
+            cache_file = cache_file_record,
+            source_format = self.source_format
         )
         
     def _prepare_metadata_access(self):
@@ -776,6 +802,12 @@ class RasdamanDataPackageWrapper(DataPackageWrapper):
         )
     }
 
+    def getSourceFormat(self): 
+        """
+        Return the source data file format. 
+        """
+        return None 
+
     def getDataStructureType(self):
         """
         Returns ``"rasdaman_array"``.
@@ -937,6 +969,13 @@ class TileIndexWrapper(RecordWrapper):
         super(TileIndexWrapper, self).__init__()
         
         self.storage_dir = None
+
+    def getSourceFormat(self): 
+        """
+        Return the source data file format. 
+        """
+        # TODO: implement proper source format for tile arrays 
+        return "image/tiff"
     
     def getType(self):
         """
