@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: eoxs_add_dataset_series.py 1656 2012-05-04 09:20:07Z schindlerf $
+# $Id$
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -44,6 +44,11 @@ from eoxserver.resources.coverages.management.commands import (
 )
 from eoxserver.resources.coverages.metadata import EOMetadata
 
+#-------------------------------------------------------------------------------
+
+from eoxserver.resources.coverages.managers import getDatasetSeriesManager
+
+#-------------------------------------------------------------------------------
 
 class Command(CommandOutputMixIn, BaseCommand):
     option_list = BaseCommand.option_list + (
@@ -52,7 +57,7 @@ class Command(CommandOutputMixIn, BaseCommand):
             default=None,
             help=('Mandatory. The EOID of the Dataset Series to be created.')
         ),
-        make_option('--data-source', '--data-sources',
+        make_option('-d','--data-source', '--data-sources',
             dest='data_sources',
             action='callback', callback=_variable_args_cb,
             default=[],
@@ -70,7 +75,7 @@ class Command(CommandOutputMixIn, BaseCommand):
                   "files. When only one is given, it is used for all data "
                   "sources.")
         ),
-        make_option('--add',
+        make_option('-a','--add',
             dest='add',
             action='callback', callback=_variable_args_cb,
             default=[],
@@ -84,21 +89,21 @@ class Command(CommandOutputMixIn, BaseCommand):
             help=("Optional. This switch explicitly turns off the "
                   "synchronization when data sources were added.")
         ),
-        make_option('--default-begin-time',
+        make_option('--begin-time','--default-begin-time',
             dest='default_begin_time',
             action="callback", callback=StringFormatCallback(getDateTime),
             default=None,
             help=("Optional. Default begin timestamp when no other EO-metadata " 
                   "is available. The format is ISO-8601.")
         ),
-        make_option('--default-end-time',
+        make_option('--end-time','--default-end-time',
             dest='default_end_time',
             action="callback", callback=StringFormatCallback(getDateTime),
             default=None,
             help=("Optional. Default end timestamp when no other EO-metadata " 
                   "is available. The format is ISO-8601.")
         ),
-        make_option('--default-footprint',
+        make_option('--footprint','--default-footprint',
             dest='default_footprint',
             action="callback", callback=StringFormatCallback(str),
             default=None,
@@ -135,7 +140,7 @@ class Command(CommandOutputMixIn, BaseCommand):
                 32.2014459999999971, 11.3610659999999992 
                 32.2014459999999971))"
     """ % ({"name": __name__.split(".")[-1]}))
-    args = '--eoid EOID'
+    args = '--eo-id EOID'
 
     def handle(self, *args, **options):
         System.init()
@@ -148,7 +153,7 @@ class Command(CommandOutputMixIn, BaseCommand):
         
         eoid = options.get("eoid")
         if eoid is None:
-            raise CommandError("Mandatory parameter `--eoid` not supplied.")
+            raise CommandError("Mandatory parameter `--eo-id` not supplied.")
         
         data_sources = options.get("data_sources", [])
         patterns = options.get("patterns", [])
@@ -171,12 +176,7 @@ class Command(CommandOutputMixIn, BaseCommand):
         # Create Dataset Series
         #=======================================================================
         
-        dss_mgr = System.getRegistry().findAndBind(
-            intf_id="resources.coverages.interfaces.Manager",
-            params={
-                "resources.coverages.interfaces.res_type": "eo.dataset_series"
-            }
-        )
+        dss_mgr = getDatasetSeriesManager() 
         
         self.print_msg("Creating Dataset Series with ID '%s'." % eoid, 1)
         
@@ -198,10 +198,8 @@ class Command(CommandOutputMixIn, BaseCommand):
                 if len(patterns) == 1:
                     patterns *= len(data_sources)
                 elif len(patterns) == 0:
-                    self.print_msg(
-                        "WARNING: No `--pattern` option given. Using " 
-                        "default.", 2
-                    )
+                    self.print_msg( "No pattern is given. "
+                                    "Using the default one.",2)
                     patterns = [None] * len(data_sources)
                 elif len(patterns) != len(data_sources):
                     raise CommandError(
