@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: mapserver.py 2127 2012-11-21 12:49:21Z meissls $
+# $Id$
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -31,24 +31,52 @@
 This module contains the abstract base classes for request handling.
 """
 
-from email.parser import Parser as MIMEParser
-from email.message import Message
+import logging
 import os.path
 from cgi import escape
+import time
 
 from django.conf import settings
-from osgeo.gdalconst import GDT_Byte, GDT_Int16, GDT_UInt16, GDT_Float32
 import mapscript
 
-from eoxserver.core.interfaces import *
+from eoxserver.core.interfaces import Method, ObjectArg, ListArg
 from eoxserver.core.registry import RegisteredInterface
+from eoxserver.core.exceptions import InternalError
+from eoxserver.contrib.gdal import GDT_Byte, GDT_Int16, GDT_UInt16, GDT_Float32 
 from eoxserver.services.base import BaseRequestHandler
-from eoxserver.services.requests import OWSRequest, Response, encode_message
+from eoxserver.services.requests import OWSRequest, Response
 from eoxserver.services.exceptions import InvalidRequestException
+from eoxserver.core.util.multiparttools import mpUnpack, mpPack, capitalize
+from eoxserver.core.util.multiparttools import getMimeType, getMultipartBoundary
+
+
+logger = logging.getLogger(__name__)
+
+#-------------------------------------------------------------------------------
+# utilities 
+
+# content-type parsing  
+is_xml       = lambda ct : getMimeType(ct) in ("text/xml","application/xml","application/gml+xml")
+is_multipart = lambda ct : getMimeType(ct).startswith("multipart/") 
+
+# capilatize header names 
+_headcap = lambda p : ( capitalize(p[0]) , p[1] ) 
+headcap  = lambda h : dict( map( _headcap , h.items() ) )  
+
+#-------------------------------------------------------------------------------
 
 class MapServerRequest(OWSRequest):
     """
-    MapServerRequest object
+    This class inherits from :class:`~.OWSRequest`.
+    
+    The constructor expects a single argument ``req`` which is expected to
+    contain an :class:`~.OWSRequest` instance. The parameters and decoder
+    will be taken from that instance.
+    
+    :class:`MapServerRequest` objects add two properties: first a ``map``
+    property which contains a :class:`mapscript.mapObj`, and second an
+    ``ows_req`` property which contains a :class:`mapscript.OWSRequest`
+    object. These properties are not configured at the beginning.
     """
     def __init__(self, req):
         super(MapServerRequest, self).__init__(req.http_req, params=req.params, decoder=req.decoder)
@@ -60,81 +88,116 @@ class MapServerRequest(OWSRequest):
 
 class MapServerResponse(Response):
     """
-    MapServerResponse object
+    This class inherits from :class:`~.Response`. It adds methods to handle
+    with response data obtained from MapServer, including methods for
+    multipart messages.
+    
+    The constructor takes several arguments. In ``ms_response``, the response
+    buffer as returned by MapServer is expected. The ``ms_content_type``
+    argument shall be set to the MIME type of the response content.
+    ``ms_status`` shall contain the MapServer status as returned by the
+    call to :meth:`mapscript.mapObj.OWSDispatch`. ``headers`` and ``status``
+    are optional and have the same meaning as in :class:`~.Response`.
     """
+    
     def __init__(self, ms_response, ms_content_type, ms_status, headers={}, status=None):
         super(MapServerResponse, self).__init__(content=ms_response, content_type=ms_content_type, headers=headers, status=status)
         self.ms_status = ms_status
         
         self.ms_response_data = None
-        self.ms_response_xml = ''
+        self.ms_response_data_headers = {} 
+        self.ms_response_xml = None 
         self.ms_response_xml_headers = {}
         
-    def _isXML(self, content_type):
-        return content_type.split(";")[0].lower() in ("text/xml", "application/xml", "application/gml+xml")
-            
-    def _isMultipart(self, content_type):
-        return content_type.split("/")[0].lower() == "multipart"
-        
     def splitResponse(self):
-        if self._isXML(self.content_type):
-            self.ms_response_xml = self.content
-            self.ms_response_xml_headers = {'Content-type': self.content_type}
-        elif self._isMultipart(self.content_type):
-            parts = MIMEParser().parsestr("Content-type:%s\n\n%s" % (self.content_type, self.content.rstrip("--wcs--\n\n"))).get_payload()
-            for part in parts:
-                if self._isXML(part.get_content_type()):
-                    self.ms_response_xml = part.get_payload()
-                    for header in part.keys():
-                        self.ms_response_xml_headers[header] = part[header]
-                else:
-                    self.ms_response_data = part
-        else:
-            self.ms_response_data = self.content
-    
-    def getProcessedResponse(self, processed_xml, processed_xml_headers=None):
-        if self.ms_response_data is not None:
-            xml_msg = Message()
-            xml_msg.set_payload(processed_xml)
-            
-            if processed_xml_headers is not None:
-                xml_headers = processed_xml_headers
-            else:
-                xml_headers = self.ms_response_xml_headers
-            for name, value in xml_headers.items():
-                xml_msg.add_header(name, value)
-            
-            if isinstance(self.ms_response_data, Message):
-                data_msg = self.ms_response_data
-            else:
-                data_msg = Message()
-                
-                data_msg.set_payload(self.ms_response_data)
-                
-                data_msg.add_header('Content-type', self.content_type)
-                for name, value in self.headers.items():
-                    data_msg.add_header(name, value)
-                
-            content = "--wcs\n%s\n--wcs\n%s\n--wcs--" % (encode_message(xml_msg), encode_message(data_msg))
-            content_type = "multipart/mixed; boundary=wcs"
-            headers = {}
-        else:
-            content = processed_xml
-            content_type = self.content_type
-            if processed_xml_headers is None:
-                headers = self.headers
-            else:
-                headers = processed_xml_headers
-            
-        return Response(content, content_type, headers, self.getStatus())
+        """
+        This method splits a multipart response into its different parts.
         
+        The XML part is stored in the ``ms_response_xml`` property of the
+        object. The coverage data is stored in the ``ms_response_data``
+        property of the object. The headers of the parts are stored in
+        the ``ms_response_xml_headers`` and ``ms_response_data_headers``
+        properties respectively.
+        """
+
+        if is_multipart( self.content_type ) : 
+        
+            # extract multipart boundary  
+            boundary = getMultipartBoundary( self.content_type ) 
+
+            for headers,offset,size in mpUnpack(self.content,boundary,capitalize=True) :
+
+                if is_xml( headers['Content-Type'] ) : 
+                    self.ms_response_xml = self.content[offset:(offset+size)]
+                    self.ms_response_xml_headers = headers
+                else : 
+                    self.ms_response_data = self.content[offset:(offset+size)] 
+                    self.ms_response_data_headers = headers
+
+        else : # single part payload 
+            
+            headers = headcap( self.headers )
+            headers['Content-Type'] = self.content_type 
+
+            if is_xml( self.content_type ) : 
+                self.ms_response_xml = self.content
+                self.ms_response_xml_headers = headers
+            else : 
+                self.ms_response_data = self.content 
+                self.ms_response_data_headers = headers
+
+    
+    def getProcessedResponse(self, response_xml, headers_xml=None, boundary="wcs", subtype="mixed"):
+        """
+        This method returns a :class:`~.Response` object that contains the
+        coverage data generated by the original MapServer call and the
+        XML data contained in the ``response_xml`` argument.
+        
+        The ``headers_xml`` parameter may contain a dictionary of headers
+        to be tagged on the XML part of the multipart response. The
+        ``boundary`` argument shall contain the boundary string used for
+        delimiting the different parts of the message and defaults to ``wcs``.
+        The ``subtype`` argument relates to the second part of the MIME type
+        statement and defaults to ``mixed`` for a complete MIME type of
+        ``multipart/mixed``.
+        """
+        if self.ms_response_data is not None: # mutipart response   
+
+            if headers_xml is None: headers_xml = self.ms_response_xml_headers
+
+            # normalize header content 
+            headers_xml  = headcap( headers_xml )
+            headers_data = headcap( self.ms_response_data_headers ) 
+
+            # prepare multipart package 
+            parts = [ 
+                  ( headers_xml.items() , response_xml ) , 
+                  ( headers_data.items() , self.ms_response_data ) ,
+                ] 
+           
+            content_type = "multipart/%s; boundary=%s"%(subtype,boundary) 
+
+            return Response(mpPack(parts,boundary),content_type,{},self.getStatus())
+ 
+        else : # pure XML response - currently not in use - possibly harmfull as there is no way to test it!  
+
+            if headers_xml is None: headers_xml = self.headers
+
+            return Response(response_xml,self.content_type,headcap(headers_xml),self.getStatus()) 
+
     def getContentType(self):
-        if "Content-type" in self.headers:
+        """
+        Returns the content type of the response.
+        """
+        if "Content-type" in self.headers: #MP: Is the 'Content-type' case correct?
             return self.headers["Content-type"]
         else:
             return self.content_type # TODO: headers for multipart messages
         
     def getStatus(self):
+        """
+        Returns the HTTP status code of the response.
+        """
         if self.status is None:
             if self.ms_status is not None and self.ms_status == 0:
                 return 200
@@ -144,6 +207,9 @@ class MapServerResponse(Response):
             return self.status
 
 class MapServerLayerGeneratorInterface(RegisteredInterface):
+    """
+    This interface is not in use.
+    """
     REGISTRY_CONF = {
         "name": "MapServer Layer Generator Interface",
         "intf_id": "services.mapserver.MapServerLayerGeneratorInterface",
@@ -162,6 +228,25 @@ class MapServerLayerGeneratorInterface(RegisteredInterface):
     )
 
 class MapServerDataConnectorInterface(RegisteredInterface):
+    """
+    This interface is intended for objects that configure the input data for
+    a MapServer layer. The basic rationale for this is that there are at
+    least three different types of data sources that need differentt
+    treatment:
+    
+    * data stored in single plain files
+    * tiled data with references in a tile index (a shape file)
+    * rasdaman arrays
+    
+    Others might be added in the course of development of EOxServer.
+    
+    .. method:: congigure(layer, eo_object)
+    
+        This method takes a :class:`mapscript.layerObj` object and an ``eo_object``
+        as input and configures the MapServer layer according to the type of
+        data package used by the ``eo_object`` (RectifiedDataset,
+        ReferenceableDataset or RectifiedStitchedMosaic).
+    """
     REGISTRY_CONF = {
         "name": "MapServer Data Connector Interface",
         "intf_id": "services.mapserver.MapServerDataConnectorInterface",
@@ -236,10 +321,6 @@ class MapServerOperationHandler(BaseRequestHandler):
         instance of ``mapscript.OWSRequest``) with the parameters
         passed in with the user request. This method can be overridden
         in order to change the treatment of parameters.
-        
-        @param  ms_req  An :class:`MapServerRequest` object
-        
-        @return         None
         """
         if self.req.getParamType() == "kvp":
             self.ows_req.type = mapscript.MS_GET_REQUEST
@@ -276,20 +357,19 @@ class MapServerOperationHandler(BaseRequestHandler):
         This method actually executes the MapServer request by calling
         ``ms_req.map.OWSDispatch()``. This method should not be
         overridden by child classes.
-        
-        @param  ms_req  An :class:`MapServerRequest` object
-        
-        @return         An :class:`MapServerResponse`
-                        object containing the content, headers and status
-                        of the request as well as the status code
-                        returned by MapServer
         """
-        logging.debug("MapServerOperationHandler.dispatch: 1")
+        
+        logger.debug("MapServer: Installing stdout to buffer.")
         mapscript.msIO_installStdoutToBuffer()
-        # Execute the OWS request by mapserver, obtain the status in dispatch_status (==0 is OK)
-        logging.debug("MapServerOperationHandler.dispatch: 2")
+        
         try:
+            logger.debug("MapServer: Dispatching.")
+            ts = time.time()
+            # Execute the OWS request by mapserver, obtain the status in 
+            # dispatch_status (0 is OK)
             dispatch_status = self.map.OWSDispatch(self.ows_req)
+            te = time.time()
+            logger.debug("MapServer: Dispatch took %f seconds." % (te - ts))
         except Exception, e:
             raise InvalidRequestException(
                 str(e),
@@ -297,15 +377,33 @@ class MapServerOperationHandler(BaseRequestHandler):
                 None
             )
         
-        logging.debug("MapServerOperationHandler.dispatch: 3")
-        content_type = mapscript.msIO_stripStdoutBufferContentType()
-        mapscript.msIO_stripStdoutBufferContentHeaders()
-        logging.debug("MapServerOperationHandler.dispatch: 4")
-        result = mapscript.msIO_getStdoutBufferBytes()
-        logging.debug("MapServerOperationHandler.dispatch: 5")
+        logger.debug("MapServer: Retrieving content-type.")
+        try:
+            content_type = mapscript.msIO_stripStdoutBufferContentType()
+            mapscript.msIO_stripStdoutBufferContentHeaders()
+
+        except mapscript.MapServerError:
+            # degenerate response. Manually split headers from content
+            complete_result = mapscript.msIO_getStdoutBufferBytes()
+            parts = complete_result.split("\r\n")
+            result = parts[-1]
+            headers = parts[:-1]
+            
+            for header in headers:
+                if header.lower().startswith("content-type"):
+                    content_type = header[14:]
+                    break
+            else:
+                content_type = None
+
+        else:
+            logger.debug("MapServer: Retrieving stdout buffer bytes.")
+            result = mapscript.msIO_getStdoutBufferBytes()
+        
+        logger.debug("MapServer: Performing MapServer cleanup.")
         # Workaround for MapServer issue #4369
         msversion = mapscript.msGetVersionInt()
-        if msversion < 60004 or ( msversion < 60200 and msversion >= 60100):
+        if msversion < 60004 or (msversion < 60200 and msversion >= 60100):
             mapscript.msCleanup()
         else:
             mapscript.msIO_resetHandlers()
@@ -317,9 +415,13 @@ class MapServerOperationHandler(BaseRequestHandler):
             try:
                 os.remove(temp_file)
             except:
-                logging.warning("Could not remove temporary file '%s'" % temp_file)
+                logger.warning("Could not remove temporary file '%s'" % temp_file)
 
 def gdalconst_to_imagemode(const):
+    """
+    This function translates a GDAL data type constant as defined in the
+    :mod:`gdalconst` module to a MapScript image mode constant.
+    """
     if const == GDT_Byte:
         return mapscript.MS_IMAGEMODE_BYTE
     elif const == GDT_Int16 or const == GDT_UInt16:
@@ -330,6 +432,11 @@ def gdalconst_to_imagemode(const):
         raise InternalError("MapServer is not capable to process the datatype %d" % const)
 
 def gdalconst_to_imagemode_string(const):
+    """
+    This function translates a GDAL data type constant as defined in the
+    :mod:`gdalconst` module to a string as used in the MapServer map file
+    to denote an image mode.
+    """
     if const == GDT_Byte:
         return "BYTE"
     elif const == GDT_Int16 or const == GDT_UInt16:
