@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: wcst11AlterCapabilities.py 1411 2012-03-02 16:40:04Z martin.paces $
+# $Id$
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Martin Paces <martin.paces@iguassu.cz>
@@ -34,6 +34,9 @@ from xml.etree import ElementTree as etree
 
 from eoxserver.core.system import System
 
+
+logger = logging.getLogger(__name__)
+
 #-------------------------------------------------------------------------------
 # actions
 
@@ -187,7 +190,7 @@ def _wcst11AlterCapabilities( respSrc , OWS ) :
     except Exception as e : 
 
         # keep track of the failures 
-        logging.debug( "_wcst11AlterCapabilities(): version %s : Content not altered! reason: %s " % ( OWS.version , str(e) ) ) 
+        logger.debug( "_wcst11AlterCapabilities(): version %s : Content not altered! reason: %s " % ( OWS.version , str(e) ) ) 
 
         # return unafected original response 
         return respSrc  
