@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: core.py 2360 2013-03-15 15:23:26Z meissls $
+# $Id$
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -42,6 +42,9 @@ from django.core.management import execute_from_command_line
 
 from eoxserver.core.system import System
 
+
+logger = logging.getLogger(__name__)
+
 BASE_FIXTURES = ["initial_rangetypes.json", "testing_base.json", "testing_asar_base.json"]
 
 class TestSchemaFactory(object):
@@ -49,13 +52,14 @@ class TestSchemaFactory(object):
     
     @classmethod
     def getSchema(cls, schema_location):
-        logging.info("Opening schema: %s" % schema_location)
+        logger.info("Opening schema: %s" % schema_location)
         f = open(schema_location)
         schema = etree.XMLSchema(etree.parse(f))
         f.close()
         
         return schema
     
+
 class EOxServerTestCase(TestCase):
     """Test are carried out in a transaction which is rolled back after each
     test."""
@@ -63,14 +67,6 @@ class EOxServerTestCase(TestCase):
     
     def setUp(self):
         System.init()
-        
-class EOxServerTransactionTestCase(TransactionTestCase):
-    """Test are not carried out in a transaction and DB data can thus be used
-    outside e.g. by MapServer."""
-    fixtures = BASE_FIXTURES
-    
-    def setUp(self):
-        System.init()
 
 class CommandTestCase(EOxServerTestCase):
     """ Base class for testing CLI tools.
@@ -90,6 +86,7 @@ class CommandTestCase(EOxServerTestCase):
         if isinstance(args, (list, tuple)):
             args.extend(self.args)
         elif isinstance(args, basestring):
+            #pylint: disable=E1101
             args.extend(self.args.split(" "))
         
         for key, value in self.kwargs.items():
@@ -176,7 +173,8 @@ def _expand_regex_method(module, klass, regex):
 
 class EOxServerTestRunner(DjangoTestSuiteRunner):
     """ 
-    Custom test runner. It extends the standard :class:`~.DjangoTestRunner` 
+    Custom test runner. It extends the standard 
+    :class:`django.test.simple.DjangoTestSuiteRunner` 
     with automatic test case search for a given regular expression.
     
     Activate by including ``TEST_RUNNER = 
